@@ -60,12 +60,6 @@ sub push_bind_values {
 }
 
 
-sub wantobjects {
-    my $self = shift;
-    return $self->{selecto};
-}
-
-
 sub arows {
     my $self = shift;
     if ($self->{arows}) {
@@ -84,15 +78,6 @@ sub acolumns {
 }
 
 
-sub column_names {
-    my $self = shift;
-    if ($self->{columns}) {
-        return map {$_->name} @{$self->{columns}};
-    }
-    return;
-}
-
-
 sub exists {
     my $self = shift;
     return SQL::DB::Expr->new('EXISTS ('. $self .')', $self->bind_values);
@@ -163,7 +148,7 @@ sub sql_insert {
 
     return "INSERT INTO\n    ". $ref->[0]->_arow->_table_name
            . ' ('
-           . join(', ', map {$_->_name} @{$ref})
+           . join(', ', map {$_->_column->name} @{$ref})
            . ")\n";
 }
 
@@ -200,9 +185,11 @@ sub sql_values {
 
 sub st_update {
     my $self = shift;
-    my $ref  = shift;
+    my $ref  = shift || croak 'update requires values';
 
     my @items = (UNIVERSAL::isa($ref,'ARRAY') ? @$ref : $ref);
+    @items || croak 'update requires values';
+
     foreach (@items) {
         if (UNIVERSAL::isa($_, 'SQL::DB::Expr')) {
             $self->push_bind_values($_->bind_values);
@@ -215,6 +202,7 @@ sub st_update {
     return;
 }
 
+
 sub sql_update {
     my $self = shift;
     my $name = shift;
@@ -222,6 +210,7 @@ sub sql_update {
     return "UPDATE\n    " . $name . "\n";
 }
 
+
 sub sql_set {
     my $self = shift;
     my $ref  = shift;
@@ -241,6 +230,8 @@ sub st_select {
     my @items    = ref($ref) eq 'ARRAY' ? @{$ref} : $ref;
     my @acolumns = map {UNIVERSAL::isa($_, 'SQL::DB::ARow') ? $_->_columns : $_} @items;
 
+    $self->push_bind_values(map {UNIVERSAL::isa($_, 'SQL::DB::Expr') ? $_->bind_values : ()} @acolumns);
+
     push(@{$self->{acolumns}}, @acolumns);
     push(@{$self->{query}}, 'sql_select', undef);
 
@@ -248,15 +239,6 @@ sub st_select {
 }
 
 
-sub st_selecto {
-    my $self = shift;
-    my $ref  = shift;
-
-    $self->{selecto} = 1;
-    return $self->st_select($ref);
-}
-
-
 sub sql_select {
     my $self = shift;
     my $ref  = shift;
@@ -458,6 +440,25 @@ sub sql_union {
 }
 
 
+sub st_intersect {
+    my $self = shift;
+    my $ref  = shift;
+    unless(UNIVERSAL::isa($ref, 'SQL::DB::Expr')) {
+        confess "Select INTERSECT must be based on SQL::DB::Expr";
+    }
+    push(@{$self->{query}}, 'sql_intersect', $ref);
+    $self->push_bind_values($ref->bind_values);
+    return;
+}
+
+
+sub sql_intersect {
+    my $self = shift;
+    my $ref  = shift;
+    return "INTERSECT \n" . $ref . "\n";
+}
+
+
 sub st_group_by {
     my $self = shift;
     my $ref  = shift;
